dnl ---------------------------------------------------------------------------
dnl FLX_CHECK_XEXTENSIONS
dnl
dnl Check for the X extension libraries Xt, Xaw and Xpm
dnl ---------------------------------------------------------------------------

AC_DEFUN(FLX_CHECK_XEXTENSIONS,
[
X_EXT_LIBS=""
if test "$have_x" = "yes"
then
  XLIB="-lXext -lX11"
  ac_save_CPPFLAGS="$CPPFLAGS"
  CPPFLAGS="$CPPFLAGS $X_CFLAGS"

  dnl *** All of the following tests require X11/Xlib.h
  AC_CHECK_HEADERS(X11/Xlib.h,
    [
      dnl *** Check for XToolkit extension
      AC_CHECK_HEADERS(X11/Intrinsic.h,
        [ 
          dnl *** If X11/Intrinsic.h exists...
          AC_CHECK_LIB(Xt, XtOpenDisplay,
          AC_DEFINE(HAVE_XTK, 1, [Define if you have the XToolkit extension]),,
          $X_LIBS -lXext -lX11 $X_EXTRA_LIBS)
        ],
        AC_MSG_WARN([[XToolkit not found, flexemu will be built without it]]),
        [#include <X11/Xlib.h>]
      )

      dnl *** Check for XPixmap extension
      AC_CHECK_HEADERS(X11/xpm.h,
        [ 
          dnl *** If X11/Xpm.h exists...
          AC_CHECK_LIB(Xpm, XpmFree,
          [AC_DEFINE(HAVE_XPM, 1, [Define if you have the Xpm extension])
           X_EXT_LIBS="$X_EXT_LIBS -lXpm"],,
          $X_LIBS -lXext -lX11 $X_EXTRA_LIBS)
        ],
        AC_MSG_WARN([[Xpm not found, flexemu will be built without it]]),
        [#include <X11/Xlib.h>]
      )
      dnl *** Check for X Athena widget extension
      AC_CHECK_HEADERS(X11/Xaw/AllWidgets.h,
        [ 
          dnl *** If X11/Xaw/AllWidgets.h exists...
          dnl *** Check for both libXaw3d and libXaw
          AC_CHECK_LIB(Xaw3d, XawToggleSetCurrent,
          [AC_DEFINE(HAVE_XAW3D, 1, [Define if you have the Xaw3d extension])
           X_EXT_LIBS="$X_EXT_LIBS -lXaw3d"],,
          $X_LIBS -lXext -lX11 $X_EXTRA_LIBS)
          AC_CHECK_LIB(Xaw, XawToggleSetCurrent,
          [AC_DEFINE(HAVE_XAW, 1, [Define if you have the Xaw extension])
           X_EXT_LIBS="$X_EXT_LIBS -lXaw"],,
          $X_LIBS -lXext -lX11 $X_EXTRA_LIBS)
        ],
        AC_MSG_WARN([[Xaw3d not found]]),
        [#include <X11/Xlib.h>]
      )
    ])
fi
AC_SUBST(X_EXT_LIBS)
])

dnl ---------------------------------------------------------------------------
dnl WX_CPP_BOOL checks whether the C++ compiler has a built in bool type
dnl
dnl call WX_CPP_BOOL - will define HAVE_BOOL if the compiler supports bool
dnl ---------------------------------------------------------------------------

AC_DEFUN(WX_CPP_BOOL,
[
  AC_CACHE_CHECK([if C++ compiler supports bool], wx_cv_cpp_bool,
  [
    AC_LANG_SAVE
    AC_LANG_CPLUSPLUS

    AC_TRY_COMPILE(
      [
      ],
      [
        bool b = true;

        return 0;
      ],
      [
        wx_cv_cpp_bool=1
      ],
      [
        wx_cv_cpp_bool=0
      ]
    )

    AC_LANG_RESTORE
  ])

  if test "x$wx_cv_cpp_bool" = x ; then
    AC_DEFINE(HAVE_BOOL,1,1)
  else
    AC_DEFINE(HAVE_BOOL,0,1)
  fi
])

dnl ---------------------------------------------------------------------------
dnl WX_PATH_WXCONFIG(VERSION, [ACTION-IF-FOUND [, ACTION-IF-NOT-FOUND]])
dnl
dnl Test for wxWindows, and define WX_CFLAGS and WX_LIBS. Set WX_CONFIG
dnl environment variable to override the default name of the wx-config script
dnl to use.
dnl ---------------------------------------------------------------------------

AC_DEFUN(WX_PATH_WXCONFIG,
[
dnl 
dnl Get the cflags and libraries from the wx-config script
dnl
AC_ARG_WITH(wx-prefix, [  --with-wx-prefix=PREFIX   Prefix where wxWindows is installed (optional)],
            wx_config_prefix="$withval", wx_config_prefix="")
AC_ARG_WITH(wx-exec-prefix,[  --with-wx-exec-prefix=PREFIX Exec prefix where wxWindows is installed (optional)],
            wx_config_exec_prefix="$withval", wx_config_exec_prefix="")

  dnl deal with optional prefixes
  if test x$wx_config_exec_prefix != x ; then
     wx_config_args="$wx_config_args --exec-prefix=$wx_config_exec_prefix"
     if test x${WX_CONFIG+set} != xset ; then
        WX_CONFIG=$wx_config_exec_prefix/bin/wx-config
     fi
  fi
  if test x$wx_config_prefix != x ; then
     wx_config_args="$wx_config_args --prefix=$wx_config_prefix"
     if test x${WX_CONFIG+set} != xset ; then
        WX_CONFIG=$wx_config_prefix/bin/wx-config
     fi
  fi

  AC_PATH_PROG(WX_CONFIG, wx-config, no)
  min_wx_version=ifelse([$1], ,2.2.1,$1)
  AC_MSG_CHECKING(for wxWindows version >= $min_wx_version)
  no_wx=""
  if test "$WX_CONFIG" = "no" ; then
    no_wx=yes
  else
    WX_CFLAGS=`$WX_CONFIG $wx_config_args --cflags`
    WX_LIBS=`$WX_CONFIG $wx_config_args --libs`
    wx_config_major_version=`$WX_CONFIG $wx_config_args --version | \
           sed 's/\([[0-9]]*\).\([[0-9]]*\).\([[0-9]]*\)/\1/'`
    wx_config_minor_version=`$WX_CONFIG $wx_config_args --version | \
           sed 's/\([[0-9]]*\).\([[0-9]]*\).\([[0-9]]*\)/\2/'`
    wx_config_micro_version=`$WX_CONFIG $wx_config_args --version | \
           sed 's/\([[0-9]]*\).\([[0-9]]*\).\([[0-9]]*\)/\3/'`
  fi

  if test "x$no_wx" = x ; then
     AC_MSG_RESULT(yes (version $wx_config_major_version.$wx_config_minor_version.$wx_config_micro_version))
     ifelse([$2], , :, [$2])     
  else
     AC_MSG_RESULT(no)
     WX_CFLAGS=""
     WX_LIBS=""
     ifelse([$3], , :, [$3])
  fi
  AC_SUBST(WX_CFLAGS)
  AC_SUBST(WX_LIBS)
])


