/*
    cpuview.cpp


    flexemu, an MC6809 emulator running FLEX
    Copyright (C) 1997-2003  W. Schwotzer

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <misc1.h>

#ifdef WIN32
#include "typedefs.h"
#include "e2video.h"
#include "mc6809.h"
#include "win32gui.h"
#include "resource.h"
#endif // WIN32

#ifdef HAVE_X11
#include <stdio.h>
#include <ctype.h>
#include "typedefs.h"
#include "e2video.h"
#include "mc6809.h"

#ifdef HAVE_XTK
#include "xtgui.h"
#include "flexcpu.xbm"
#endif
#endif

#include "schedule.h"
#include "clogfile.h"

#ifdef WIN32
extern Win32Gui *wgui;
#endif

/***************************************/
/* CPU-view with XToolkit interface    */
/***************************************/

#ifdef HAVE_XTK

extern int ok_width;

int radio_data[] = {S_NO_CHANGE, S_RUN, S_STOP, S_STEP, S_EXIT, S_RESET, S_NEXT};

void XtGui::popup_cpu(void)
{
        XtPopup(cpuframe, XtGrabNone);
        if (okpixmap != None)
                XtVaSetValues(entry24, XtNleftBitmap, okpixmap, NULL);
	cpu_popped_up = true;
}

void XtGui::popdown_cpu(void)
{
        XtVaSetValues(entry24, XtNleftBitmap, None, NULL);
	XtPopdown(cpuframe);
	cpu_popped_up = false;
}

void XtGui::toggle_cpu(void)
{
	if (cpu_popped_up)
		popdown_cpu();
	else
		popup_cpu();
}

void XtGui::popup_bp()
{
	String bpstring;
	unsigned int which;

	bpstring = (String)XtMalloc(16);
	for (which = 0; which <= 1; which++)
	{
		if (!cpu->is_bp_set(which))
			strcpy(bpstring, "");
		else
			sprintf(bpstring, "%04x", cpu->get_bp(which));
		XtVaSetValues(bptext[which], XtNlength, 6,
			XtNstring, (XtArgVal)bpstring, NULL);
	}
	XtPopup(bpframe, XtGrabExclusive);
	XtFree(bpstring);
}

void XtGui::popdown_bp(Widget w)
{
	String bpstring;
	unsigned int addr, which;

	if (w != bpokbutton)
	{
		XtPopdown(bpframe);
		return;
	}

	for (which = 0; which <= 1; which++)
	{
		XtVaGetValues(bptext[which], XtNstring, &bpstring, NULL);
		if (sscanf(bpstring, "%x", (int *)&addr) == 1 && addr <= 0xffff)
			cpu->set_bp(which, (Word)addr);
		else
			if (strlen(bpstring) == 0)
		 		cpu->reset_bp(which);
	}
	XtPopdown(bpframe);
}

void XtGui::clear_bp()
{
	unsigned int which;

	for (which = 0; which <= 1; which++)
		XtVaSetValues(bptext[which], XtNstring, (XtArgVal)"", NULL);
}

void XtGui::popup_log()
{
	String tmpstring = (String)XtMalloc(PATH_MAX);

	int which = 0;
	// Min Address
	tmpstring[0] = '\0';
	if (lfs.minAddr < 0x10000)
		sprintf(tmpstring, "%04x", lfs.minAddr);
	XtVaSetValues(logtext[which++], XtNlength, 6,
		XtNstring, (XtArgVal)tmpstring, NULL);
	// Max Address
	tmpstring[0] = '\0';
	if (lfs.maxAddr < 0x10000)
		sprintf(tmpstring, "%04x", lfs.maxAddr);
	XtVaSetValues(logtext[which++], XtNlength, 6,
		XtNstring, (XtArgVal)tmpstring, NULL);
	// Start Address
	tmpstring[0] = '\0';
	if (lfs.startAddr < 0x10000)
		sprintf(tmpstring, "%04x", lfs.startAddr);
	XtVaSetValues(logtext[which++], XtNlength, 6,
		XtNstring, (XtArgVal)tmpstring, NULL);
	// Stop Address
	tmpstring[0] = '\0';
	if (lfs.stopAddr < 0x10000)
		sprintf(tmpstring, "%04x", lfs.stopAddr);
	XtVaSetValues(logtext[which++], XtNlength, 6,
		XtNstring, (XtArgVal)tmpstring, NULL);
	// Log Filename
	strcpy(tmpstring, lfs.logFileName);
	XtVaSetValues(logfilename, XtNlength, 6,
		XtNstring, (XtArgVal)tmpstring, NULL);

	XtPopup(logframe, XtGrabExclusive);
	XtFree(tmpstring);
}

void XtGui::popdown_log(Widget w)
{
	String tmpstring;
	unsigned int addr, which;

	if (w != logokbutton)
	{
		XtPopdown(logframe);
		return;
	}

	which = 0;
	// Min Address
	XtVaGetValues(logtext[which++], XtNstring, &tmpstring, NULL);
	lfs.minAddr = 0x0000;
	if (sscanf(tmpstring, "%x", (int *)&addr) == 1 && addr <= 0xffff)
		lfs.minAddr = addr;
	// Max Address
	XtVaGetValues(logtext[which++], XtNstring, &tmpstring, NULL);
	lfs.maxAddr = 0xFFFF;
	if (sscanf(tmpstring, "%x", (int *)&addr) == 1 && addr <= 0xffff)
		lfs.maxAddr = addr;
	// Start Address
	XtVaGetValues(logtext[which++], XtNstring, &tmpstring, NULL);
	lfs.startAddr = 0x10000;
	if (sscanf(tmpstring, "%x", (int *)&addr) == 1 && addr <= 0xffff)
		lfs.startAddr = addr;
	// Stop Address
	XtVaGetValues(logtext[which++], XtNstring, &tmpstring, NULL);
	lfs.stopAddr = 0x10000;
	if (sscanf(tmpstring, "%x", (int *)&addr) == 1 && addr <= 0xffff)
		lfs.stopAddr = addr;
	// Log Filename
	XtVaGetValues(logfilename, XtNstring, &tmpstring, NULL);
	strncpy(lfs.logFileName, tmpstring, PATH_MAX);
	lfs.logFileName[PATH_MAX-1] = '\0';

	schedy->sync_exec(new CSetLogFile(*cpu, &lfs));

	XtPopdown(logframe);
}

void XtGui::clear_log()
{
	unsigned int which;

	for (which = 0; which <= 3; which++)
		XtVaSetValues(logtext[which], XtNstring, (XtArgVal)"", NULL);
	XtVaSetValues(logfilename, XtNstring, (XtArgVal)"", NULL);
}

void XtGui::create_cpuview(Widget parent)
{
	cpu_popped_up = false;
	cpuframe = XtVaCreatePopupShell("MC6809",
		topLevelShellWidgetClass, parent, NULL);
	cpuform = XtVaCreateManagedWidget("cpuForm",
		formWidgetClass, cpuframe, NULL);
	cputext = XtVaCreateManagedWidget("cpuText",
		asciiTextWidgetClass, cpuform, NULL);
	runbutton = XtVaCreateManagedWidget("runButton",
		toggleWidgetClass, cpuform,
		XtNradioData, (XtArgVal)&radio_data[S_RUN],
		XtNstate, (XtArgVal)1, NULL);
	stopbutton = XtVaCreateManagedWidget("stopButton",
		toggleWidgetClass, cpuform,
		XtNradioGroup, (XtArgVal)runbutton,
		XtNradioData, (XtArgVal)&radio_data[S_STOP], NULL);
	stepbutton = XtVaCreateManagedWidget("stepButton",
		toggleWidgetClass, cpuform,
		XtNradioGroup, (XtArgVal)runbutton,
		XtNradioData, (XtArgVal)&radio_data[S_STEP], NULL);
	nextbutton = XtVaCreateManagedWidget("nextButton",
		toggleWidgetClass, cpuform,
		XtNradioGroup, (XtArgVal)runbutton,
		XtNradioData, (XtArgVal)&radio_data[S_NEXT], NULL);
	resetbutton = XtVaCreateManagedWidget("resetButton",
		toggleWidgetClass, cpuform,
		XtNradioGroup, (XtArgVal)runbutton,
		XtNradioData, (XtArgVal)&radio_data[S_RESET], NULL);
	bpbutton = XtVaCreateManagedWidget("bpButton",
		commandWidgetClass, cpuform, NULL);
	logbutton = XtVaCreateManagedWidget("logButton",
		commandWidgetClass, cpuform, NULL);
	cpubutton = XtVaCreateManagedWidget("cpuButton",
		commandWidgetClass, cpuform, NULL);

	cpupixmap = XCreateBitmapFromData(XtDisplay(cpuframe),
		RootWindowOfScreen(XtScreen(cpuframe)),
		flexcpu_bits, flexcpu_width, flexcpu_height);
	if (cpupixmap != None)
		XtVaSetValues(cpuframe, XtNiconPixmap, cpupixmap, NULL);

	XtAddCallback(runbutton,   XtNcallback, setCpuRunCallback,
		(XtPointer)this);
	XtAddCallback(stopbutton,  XtNcallback, setCpuStopCallback,
		(XtPointer)this);
	XtAddCallback(stepbutton,  XtNcallback, setCpuStepCallback,
		(XtPointer)this);
	XtAddCallback(nextbutton,  XtNcallback, setCpuNextCallback,
		(XtPointer)this);
	XtAddCallback(resetbutton, XtNcallback, setCpuResetCallback,
		(XtPointer)this);
	XtAddCallback(bpbutton,    XtNcallback, popupBpCallback,
		(XtPointer)this);
	XtAddCallback(logbutton,   XtNcallback, popupLogfileCallback,
		(XtPointer)this);
	XtAddCallback(cpubutton,   XtNcallback, popdownCpuCallback,
		(XtPointer)this);
} // create_cpuview

void XtGui::create_bp_dialog(Widget parent)
{
	bpframe = XtVaCreatePopupShell("Breakpoints",
		transientShellWidgetClass, parent, NULL);
	bpform = XtVaCreateManagedWidget("bpForm",
		formWidgetClass, bpframe, NULL);
	bplabel[0] = XtVaCreateManagedWidget("bpLabel1",
		labelWidgetClass, bpform, NULL);
	bplabel[1] = XtVaCreateManagedWidget("bpLabel2",
		labelWidgetClass, bpform, NULL);
	bptext[0] = XtVaCreateManagedWidget("bpText1",
		asciiTextWidgetClass, bpform,
		XtNeditType, (XtArgVal)XawtextEdit, NULL);
	bptext[1] = XtVaCreateManagedWidget("bpText2",
		asciiTextWidgetClass, bpform,
		XtNeditType, (XtArgVal)XawtextEdit, NULL);
	bpokbutton = XtVaCreateManagedWidget("bpOkButton",
		commandWidgetClass, bpform, NULL);
	bpcancelbutton = XtVaCreateManagedWidget("bpCancelButton",
		commandWidgetClass, bpform, NULL);
	bpclearbutton = XtVaCreateManagedWidget("bpClearButton",
		commandWidgetClass, bpform, NULL);

	XtAddCallback(bpokbutton, XtNcallback, popdownBpCallback,
		(XtPointer)this);
	XtAddCallback(bpcancelbutton, XtNcallback, popdownBpCallback,
		(XtPointer)this);
	XtAddCallback(bpclearbutton, XtNcallback, clearBpCallback,
		(XtPointer)this);

	XtInstallAllAccelerators(bpform, bptext[0]);
} // create_bp_dialog

void XtGui::create_logfile_dialog(Widget parent)
{
	logframe = XtVaCreatePopupShell("Logging",
		transientShellWidgetClass, parent, NULL);
	logform = XtVaCreateManagedWidget("logForm",
		formWidgetClass, logframe, NULL);
	loglabel[0] = XtVaCreateManagedWidget("logLabel1",
		labelWidgetClass, logform, NULL);
	loglabel[1] = XtVaCreateManagedWidget("logLabel2",
		labelWidgetClass, logform, NULL);
	loglabel[2] = XtVaCreateManagedWidget("logLabel3",
		labelWidgetClass, logform, NULL);
	loglabel[3] = XtVaCreateManagedWidget("logLabel4",
		labelWidgetClass, logform, NULL);
	loglabel[4] = XtVaCreateManagedWidget("logLabel5",
		labelWidgetClass, logform, NULL);
	logtext[0] = XtVaCreateManagedWidget("logText1",
		asciiTextWidgetClass, logform,
		XtNeditType, (XtArgVal)XawtextEdit, NULL);
	logtext[1] = XtVaCreateManagedWidget("logText2",
		asciiTextWidgetClass, logform,
		XtNeditType, (XtArgVal)XawtextEdit, NULL);
	logtext[2] = XtVaCreateManagedWidget("logText3",
		asciiTextWidgetClass, logform,
		XtNeditType, (XtArgVal)XawtextEdit, NULL);
	logtext[3] = XtVaCreateManagedWidget("logText4",
		asciiTextWidgetClass, logform,
		XtNeditType, (XtArgVal)XawtextEdit, NULL);
	logfilename = XtVaCreateManagedWidget("logFileName",
		asciiTextWidgetClass, logform,
		XtNeditType, (XtArgVal)XawtextEdit, NULL);
	logokbutton = XtVaCreateManagedWidget("logOkButton",
		commandWidgetClass, logform, NULL);
	logcancelbutton = XtVaCreateManagedWidget("logCancelButton",
		commandWidgetClass, logform, NULL);
	logclearbutton = XtVaCreateManagedWidget("logClearButton",
		commandWidgetClass, logform, NULL);

	XtAddCallback(logokbutton, XtNcallback, popdownLogCallback,
		(XtPointer)this);
	XtAddCallback(logcancelbutton, XtNcallback, popdownLogCallback,
		(XtPointer)this);
	XtAddCallback(logclearbutton, XtNcallback, clearLogCallback,
		(XtPointer)this);

	XtInstallAllAccelerators(logform, logtext[0]);
} // create_logfile_dialog

void XtGui::redraw_cpuview_impl(const s_cpu_status &stat)
{
	int i;

	i = stat.s & 7;
	text(5 + 3 * i, 10, "[");
	text(8 + 3 * i, 10, "]");
	XtVaSetValues(cputext, XtNlength, strlen(cpustring)+1,
			XtNstring, cpustring, NULL);
	if (stat.state == S_RESET_RUN)
		XawToggleSetCurrent(runbutton, (XtPointer)&radio_data[S_RESET]);
	else if (stat.state == S_INVALID)
		XawToggleSetCurrent(runbutton, (XtPointer)&radio_data[S_STOP]);
	else
		XawToggleSetCurrent(runbutton, (XtPointer)&radio_data[stat.state]);
	if (stat.state == S_RUN) {
		XtVaSetValues(stepbutton, XtNsensitive, 0, NULL);
		XtVaSetValues(nextbutton, XtNsensitive, 0, NULL);
		XtVaSetValues(resetbutton, XtNsensitive, 0, NULL);
	} else {
		XtVaSetValues(stepbutton, XtNsensitive, 1, NULL);
		XtVaSetValues(nextbutton, XtNsensitive, 1, NULL);
		XtVaSetValues(resetbutton, XtNsensitive, 1, NULL);
	}
}
#endif // #ifdef HAVE_XTK

/***************************************/
/* CPU-view with Win32 interface       */
/***************************************/

#ifdef WIN32

extern int ok_width;

int radio_data[] = {S_NO_CHANGE, S_RUN, S_STOP, S_STEP, S_EXIT, S_RESET, S_NEXT};

void Win32Gui::popup_cpu(void)
{
	CheckMenuItem(menu2, IDM_VIEW, MF_BYCOMMAND | MF_CHECKED); 
	ShowWindow(cpuform, SW_SHOW);
	cpu_popped_up = true;
}

void Win32Gui::popdown_cpu(void)
{
	CheckMenuItem(menu2, IDM_VIEW, MF_BYCOMMAND | MF_UNCHECKED); 
	ShowWindow(cpuform, SW_HIDE);
	cpu_popped_up = false;
}

void Win32Gui::toggle_cpu(void)
{
	if (cpu_popped_up)
	{
		popdown_cpu();
	} else {
		popup_cpu();
	}
}

LOGFONT *Win32Gui::getLogFontStruct(HDC hdc, int pointSize)
{
	static LOGFONT lf;

	lf.lfHeight		= -MulDiv(pointSize,
				  GetDeviceCaps(hdc, LOGPIXELSY), 72);
	lf.lfWidth		= 0;
	lf.lfEscapement		= 0;
	lf.lfOrientation	= 0;
	lf.lfWeight		= FW_DONTCARE;
	lf.lfItalic		= 0;
	lf.lfUnderline		= 0;
	lf.lfStrikeOut		= 0;
	lf.lfCharSet		= ANSI_CHARSET;
	lf.lfOutPrecision	= OUT_TT_PRECIS;
	lf.lfClipPrecision	= CLIP_DEFAULT_PRECIS;
	lf.lfQuality		= DEFAULT_QUALITY;
	lf.lfPitchAndFamily	= FIXED_PITCH | FF_DONTCARE;
	lf.lfFaceName[0]	= '\0';
	return &lf;
} // getLogFontStruct

void Win32Gui::create_cpuview(HWND parent, struct sGuiOptions *pOptions)
{
	DWORD	style;
	HDC	hdc;
	RECT	rect;

	cpu_popped_up = false;
	rect.left	= (long)0;
	rect.right	= (long)CPUW;
	rect.top	= (long)0;
	rect.bottom	= (long)CPUH;
	AdjustWindowRectEx(&rect, WS_OVERLAPPEDWINDOW, TRUE, 0);
	cpuform = CreateWindowEx (0,
		"Mc6809",
		"MC6809",
		WS_OVERLAPPEDWINDOW,
		CW_USEDEFAULT,
		0,
		rect.right - rect.left,
		rect.bottom - rect.top,
		parent,
		NULL, 
		pOptions->hInstance,
		NULL);
	style = WS_CHILD | WS_VISIBLE;
	runbutton = CreateWindow("button", "&Run", style,     BL, BT,     BW, BH,
		cpuform, (HMENU)IDP_RUN, pOptions->hInstance, NULL);
	stopbutton = CreateWindow("button", "S&top", style,   BL, BT+24,  BW, BH,
		cpuform, (HMENU)IDP_STOP, pOptions->hInstance, NULL); 
	stepbutton = CreateWindow("button", "&Step", style,   BL, BT+48,  BW, BH,
		cpuform, (HMENU)IDP_STEP, pOptions->hInstance, NULL);
	nextbutton = CreateWindow("button", "&Next", style,   BL, BT+72,  BW, BH,
		cpuform, (HMENU)IDP_NEXT, pOptions->hInstance, NULL);
	resetbutton = CreateWindow("button", "R&eset", style, BL, BT+96,  BW, BH,
		cpuform, (HMENU)IDP_RESET, pOptions->hInstance, NULL);
	bpbutton = CreateWindow("button", "&BrkPnts", style,  BL, BT+120, BW, BH,
		cpuform, (HMENU)IDP_BP, pOptions->hInstance, NULL);
	logbutton = CreateWindow("button", "&LogFile", style, BL, BT+144, BW, BH,
		cpuform, (HMENU)IDP_LOG, pOptions->hInstance, NULL);
	style |= BS_DEFPUSHBUTTON;
	cpubutton = CreateWindow("button", "&OK", style,      BL, BT+168, BW, BH,
		cpuform, (HMENU)IDP_CPU, pOptions->hInstance, NULL);
	// change the font to a fixed font
	hdc = GetDC(cpuform);
	hFontFixed = CreateFontIndirect(getLogFontStruct(hdc, 8));
	SetWindowFont(cpuform,     hFontFixed, TRUE);
	SetWindowFont(nextbutton,  hFontFixed, TRUE);
	SetWindowFont(stepbutton,  hFontFixed, TRUE);
	SetWindowFont(runbutton,   hFontFixed, TRUE);
	SetWindowFont(stopbutton,  hFontFixed, TRUE);
	SetWindowFont(resetbutton, hFontFixed, TRUE);
	SetWindowFont(bpbutton,    hFontFixed, TRUE);
	SetWindowFont(logbutton,   hFontFixed, TRUE);
	SetWindowFont(cpubutton,   hFontFixed, TRUE);
	ReleaseDC(cpuform, hdc);
	// Accelerators
	hAccel = LoadAccelerators(hInstance, MAKEINTRESOURCE(IDR_CPU_ACCELERATOR));
	hAccelHwnd = cpuform;
}  // create_cpuview

BOOL Win32Gui::onCpuCommand(HWND hwnd, int cmd)
{
	switch (cmd) {
		// cpuview button controls
		case IDP_NEXT:	set_new_state(S_NEXT);		break;
		case IDP_STEP:	set_new_state(S_STEP);		break;
		case IDP_STOP:	set_new_state(S_STOP);		break;
		case IDP_RUN:	set_new_state(S_RUN);		break;
		case IDP_RESET:	set_new_state(S_RESET);		break;
		case IDP_BP:	popup_bp(hwnd);				break;
		case IDP_LOG:	popup_log(hwnd);			break;
		case IDP_CPU:	popdown_cpu();				break;
		default:	return FALSE;
	}
	return TRUE;
} // onCpuCommand


BOOL Win32Gui::onCpuPaint(HWND hwnd)
{
	redraw_cpuview(cpu_stat);
	return TRUE;
}

BOOL Win32Gui::onCpuClose(HWND hwnd)
{
	popdown_cpu();
	return TRUE;
}

LRESULT CALLBACK cpuWindowWndProc(
	HWND hwnd,
	UINT message,
	WPARAM wParam,
	LPARAM lParam)
{
	switch(message) {
		case WM_PAINT:	return wgui->onCpuPaint(hwnd);
		case WM_COMMAND:return wgui->onCpuCommand(hwnd,LOWORD(wParam));
		case WM_CLOSE:	return wgui->onCpuClose(hwnd);
		default:		return DefWindowProc(hwnd, message, wParam, lParam);
	} // switch
	return FALSE;
} // cpuWindowWndProc

void Win32Gui::redraw_cpuview_impl(const s_cpu_status &stat)
{
	int			i;
	HDC			hdc;
	PAINTSTRUCT	ps;
	HWND		hwnd;
	HFONT		hPrevFont;
	RECT		rect;

	hwnd = cpuform;
	i = stat.s & 7;
	text(5 + 3 * i, 10, "[");
	text(8 + 3 * i, 10, "]");
	rect.left	= CPUFRAME;
	rect.top	= CPUFRAME;
	rect.right	= BL-CPUFRAME;
	rect.bottom	= CPUH+2*CPUFRAME;
	hdc = BeginPaint(hwnd, &ps);
	hPrevFont = SelectFont(hdc, hFontFixed);
	DrawText(hdc, (char *)&cpustring, strlen(cpustring),
		&rect, DT_LEFT | DT_NOPREFIX);
	SelectFont(hdc, hPrevFont);
	MoveToEx(hdc, rect.left-2, rect.top-2, NULL);
	LineTo(hdc, rect.right+2, rect.top-2);
	LineTo(hdc, rect.right+2, rect.bottom+2);
	LineTo(hdc, rect.left-2, rect.bottom+2);
	LineTo(hdc, rect.left-2, rect.top-2);
	EndPaint(hwnd, &ps);
} // redraw_cpuview_impl

void Win32Gui::update_cpuview(const s_cpu_status &)
{
	RedrawWindow(cpuform, NULL, NULL, RDW_INVALIDATE);
}

BOOL Win32Gui::onBpPaint(HWND hwnd)
{
//	switch (cmd) {
//	}
	return FALSE;
} // onBpPaint

BOOL Win32Gui::onBpCommand(HWND hwnd, int cmd)
{
	switch (cmd) {
		// BP-Dialog button controls
		case IDC_BP_OK:		popdown_bp(cmd, hwnd); break;
		case IDC_BP_CANCEL:	popdown_bp(cmd, hwnd); break;
		case IDC_BP_CLEAR:	clear_bp(hwnd);        break;
		default:		return FALSE;
	}
	return TRUE;
} // onBpCommand

BOOL Win32Gui::onBpInit(HWND hwnd)
{
	char bpstring[16];
	HWND hEdit;
	int which;

	sprintf(bpstring, "Breakpoints");
	SetWindowText(hwnd, bpstring);

	for (which = 0; which <= 1; which++)
	{
		strcpy(bpstring, "");
		if (cpu->is_bp_set(which))
			sprintf(bpstring, "%04x", cpu->get_bp(which));
		SetDlgItemText(hwnd, IDC_BP_ADDR1 + which, bpstring);
	}

	if ((hEdit = GetDlgItem(hwnd, IDC_BP_ADDR1))) {
		// Select contents of edit control
		SendMessage(hEdit, EM_SETSEL, 0, -1);
	}
	return TRUE;
}

BOOL Win32Gui::onBpClose(HWND hwnd)
{
	EndDialog(hwnd, 0);
	return TRUE;
}


BOOL CALLBACK bpDialogProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch(message) {
		case WM_INITDIALOG:
			return wgui->onBpInit(hwnd);
		case WM_PAINT:
			return wgui->onBpPaint(hwnd);
		case WM_COMMAND:
			return wgui->onBpCommand(hwnd,LOWORD(wParam));
		case WM_CLOSE:
			return wgui->onBpClose(hwnd);
	} // switch
	return FALSE;
} // bpDialogProc

void Win32Gui::popup_bp(HWND hwnd)
{
	int  res;

	res = DialogBox(hInstance, MAKEINTRESOURCE(IDD_BP_DIALOG), hwnd, CALLBACKCAST bpDialogProc);
}

void Win32Gui::popdown_bp(int cmd, HWND hwnd)
{
	char		bpstring[16];
	unsigned int	addr, which;

	if (cmd == IDC_BP_OK)
		for (which = 0; which <= 1; which++)
		{
			GetDlgItemText(hwnd, IDC_BP_ADDR1 + which, bpstring, 16);
			if (sscanf(bpstring, "%x", (int *)&addr) == 1 && addr <= 0xffff)
				cpu->set_bp(which, (Word)addr);
			else
				if (strlen(bpstring) == 0)
		 			cpu->reset_bp(which);
		}

	EndDialog(hwnd, 0);
}

void Win32Gui::clear_bp(HWND hwnd)
{
	SetDlgItemText(hwnd, IDC_BP_ADDR1, "");
	SetDlgItemText(hwnd, IDC_BP_ADDR2, "");
}

BOOL Win32Gui::onLogCommand(HWND hwnd, int cmd)
{
	switch (cmd) {
		// Log-Dialog button controls
		case IDC_LOG_OK:		popdown_log(cmd, hwnd); break;
		case IDC_LOG_CANCEL:	popdown_log(cmd, hwnd); break;
		case IDC_LOG_CLEAR:		clear_log(hwnd);        break;
		case IDC_LOG_FPROMPT:	prompt_logfile(hwnd);   break;
		default:		return FALSE;
	}
	return TRUE;
} // onLogCommand

BOOL Win32Gui::onLogInit(HWND hwnd)
{
	HWND hEdit;
	char tmpstring[32];

	sprintf(tmpstring, "Logging");
	SetWindowText(hwnd, tmpstring);

	// Min Address
	tmpstring[0] = '\0';
	if (lfs.minAddr < 0x10000)
		sprintf(tmpstring, "%04X", lfs.minAddr);
	SetDlgItemText(hwnd, IDC_LOG_MINADDR, tmpstring);
	// Max Address
	tmpstring[0] = '\0';
	if (lfs.maxAddr < 0x10000)
		sprintf(tmpstring, "%04X", lfs.maxAddr);
	SetDlgItemText(hwnd, IDC_LOG_MAXADDR, tmpstring);
	// Start Address
	tmpstring[0] = '\0';
	if (lfs.startAddr < 0x10000)
		sprintf(tmpstring, "%04X", lfs.startAddr);
	SetDlgItemText(hwnd, IDC_LOG_STARTADDR, tmpstring);
	// Stop Address
	tmpstring[0] = '\0';
	if (lfs.stopAddr < 0x10000)
		sprintf(tmpstring, "%04X", lfs.stopAddr);
	SetDlgItemText(hwnd, IDC_LOG_STOPADDR, tmpstring);
	SetDlgItemText(hwnd, IDC_LOG_FILENAME, lfs.logFileName);

	if ((hEdit = GetDlgItem(hwnd, IDC_LOG_MINADDR))) {
		// Select contents of edit control
		SendMessage(hEdit, EM_SETSEL, 0, -1);
	}
	return TRUE;
}

BOOL Win32Gui::onLogClose(HWND hwnd)
{
	EndDialog(hwnd, 0);
	return TRUE;
}

BOOL CALLBACK logDialogProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch(message) {
		case WM_INITDIALOG:
			return wgui->onLogInit(hwnd);
		case WM_COMMAND:
			return wgui->onLogCommand(hwnd,LOWORD(wParam));
		case WM_CLOSE:
			return wgui->onLogClose(hwnd);
	} // switch
	return FALSE;
} // logDialogProc

void Win32Gui::popup_log(HWND hwnd)
{
	int  res;

	res = DialogBox(hInstance, MAKEINTRESOURCE(IDD_LOG_DIALOG), hwnd, CALLBACKCAST logDialogProc);
}

void Win32Gui::popdown_log(int cmd, HWND hwnd)
{
	char		tmpstring[PATH_MAX];
	unsigned int	addr;

	if (cmd == IDC_LOG_OK)
	{
		// Min Address
		GetDlgItemText(hwnd, IDC_LOG_MINADDR, tmpstring, 16);
		lfs.minAddr = 0x0000;
		if (sscanf(tmpstring, "%x", (int *)&addr) == 1 && addr <= 0xffff)
			lfs.minAddr = addr;
		// Max Address
		GetDlgItemText(hwnd, IDC_LOG_MAXADDR, tmpstring, 16);
		lfs.maxAddr = 0xFFFF;
		if (sscanf(tmpstring, "%x", (int *)&addr) == 1 && addr <= 0xffff)
			lfs.maxAddr = addr;
		// Start Address
		GetDlgItemText(hwnd, IDC_LOG_STARTADDR, tmpstring, 16);
		lfs.startAddr = 0x10000;
		if (sscanf(tmpstring, "%x", (int *)&addr) == 1 && addr <= 0xffff)
			lfs.startAddr = addr;
		// Stop Address
		GetDlgItemText(hwnd, IDC_LOG_STOPADDR, tmpstring, 16);
		lfs.stopAddr = 0x10000;
		if (sscanf(tmpstring, "%x", (int *)&addr) == 1 && addr <= 0xffff)
			lfs.stopAddr = addr;
		// Log Filename
		GetDlgItemText(hwnd, IDC_LOG_FILENAME, tmpstring, PATH_MAX);
		strncpy(lfs.logFileName, tmpstring, PATH_MAX);
		lfs.logFileName[PATH_MAX-1] = '\0';

		schedy->sync_exec(new CSetLogFile(*cpu, &lfs));
	}

	EndDialog(hwnd, 0);
}

void Win32Gui::clear_log(HWND hwnd)
{
	SetDlgItemText(hwnd, IDC_LOG_MINADDR,   "");
	SetDlgItemText(hwnd, IDC_LOG_MAXADDR,   "");
	SetDlgItemText(hwnd, IDC_LOG_STARTADDR, "");
	SetDlgItemText(hwnd, IDC_LOG_STOPADDR,  "");
	SetDlgItemText(hwnd, IDC_LOG_FILENAME,  "");
}

void Win32Gui::prompt_logfile(HWND hwnd)
{
	OPENFILENAME of;
	char filename[PATH_MAX];

	GetDlgItemText(hwnd, IDC_LOG_FILENAME, filename, PATH_MAX);
	memset(&of, 0, sizeof(OPENFILENAME));
	of.lStructSize = sizeof(OPENFILENAME);
	of.hwndOwner   = hwnd;
	of.hInstance   = hInstance;
	of.lpstrFilter = "*.LOG";
	of.lpstrFile   = filename;
	of.nMaxFile    = PATH_MAX;
	of.lpstrTitle  = "Choose Logging Filename";
	of.Flags       = OFN_HIDEREADONLY;
	of.lpstrDefExt = "LOG";

	if (GetSaveFileName(&of))
	{
		SetDlgItemText(hwnd, IDC_LOG_FILENAME, of.lpstrFile);
	}
}

#endif // WIN32
