/*
    xtgui.cpp  user interface for XToolkit


    flexemu, an MC6809 emulator running FLEX
    Copyright (C) 1997-2003  W. Schwotzer

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <misc1.h>

#ifdef HAVE_XTK

#include <ctype.h>
#include "xtgui.h"
#include "benv.h"
#include "csetfreq.h"
#include "e2video.h"
#include "mc6809.h"
#include "inout.h"
#include "schedule.h"

#include "ok.xbm"
#include "flexmain.xbm"

#ifdef HAVE_XPM
#include "author.xpm"
#endif

// global variables
XtGui *ggui;

const char *XtGui::atomNames[maxAtoms] =
{
  "WM_PROTOCOLS",
  "WM_TAKE_FOCUS",
  NULL, NULL, NULL, NULL, NULL,
  NULL, NULL, NULL
};
typedef enum 
{
   FLX_WM_PROTOCOLS         = 0,
   FLX_WM_TAKE_FOCUS        = 1
} tAtomIds;

/*ARGSUSED*/
void expose(Widget w, XEvent *pevent, String *pparams, Cardinal *pnparams)
{
	ggui->c_expose(pevent);
}

#ifdef HAVE_XPM
/*ARGSUSED*/
void author_expose(Widget w, XEvent *pevent, String *pparams, Cardinal *pnparams)
{
	ggui->a_expose(w, pevent);
}
#endif

/*ARGSUSED*/
void keyPress(Widget w, XEvent *pevent, String *pparams, Cardinal *pnparams)
{
	ggui->c_keyPress(pevent);
}

/*ARGSUSED*/
void buttonPress(Widget w, XEvent *pevent, String *pparams, Cardinal *pnparams)
{
	ggui->c_buttonPress(pevent);
}

/*ARGSUSED*/
void focusInMain(Widget w, XEvent *pevent, String *pparams, Cardinal *pnparams)
{
	ggui->c_focusIn(pevent);
}


/*ARGSUSED*/
void popup_no_resources(Widget w, XEvent *pevent, String *pparams,
	Cardinal *pnparams)
{
	ggui->popup_message("");
	ggui->set_exit();
}

/*ARGSUSED*/
void processResize(Widget w, XEvent *pevent, String *pparams,
	Cardinal *pnparams)
{
	ggui->c_process_resize(pevent);
}

/*ARGSUSED*/
void clientMsg(Widget w, XEvent *pevent, String *pparams, Cardinal *pnparams)
{
	ggui->c_clientMsg(pevent);
}

void XtGui::timerCallbackProc(XtPointer p, XtIntervalId *pId)
{
	if (p != NULL)
	{
		XtGui *pGui = (XtGui *)p;
		pGui->timerCallback(*pId);
	}
}

void XtGui::initialize(struct sGuiOptions *pOptions)
{
	XAbstractGui::initialize(pOptions);
	initialize_xrad_file();
	lfs.logFileName[0] = '\0';
	lfs.minAddr   = 0x0000;
	lfs.maxAddr   = 0xFFFF;
	lfs.startAddr = 0x10000;
	lfs.stopAddr  = 0x10000;
	ggui = this;		// global instance pointer needed for callbacks
	initialize_e2window(pOptions);
} // initialize

void XtGui::initialize_xrad_file()
{
	BEnvironment env;
	BString      adFileName;

	if (env.GetValue("HOME", adFileName))
	{
		BString defaultAdFileName;
		FILE *ifp, *ofp;

		adFileName += PATHSEPARATORSTRING PACKAGE;
		defaultAdFileName = DATADIR PATHSEPARATORSTRING PACKAGE ".ad";
		if (access(adFileName, F_OK) == -1 &&
		    ((ifp = fopen(defaultAdFileName, "r")) != NULL) &&
		    ((ofp = fopen(adFileName, "w")) != NULL))
		{
			char buf[128];

			while (fgets(buf, 128, ifp))
			   fputs(buf, ofp);
			fclose(ifp);
			fclose(ofp);
		}
	}
} // initialize_xrad_file

/*ARGSUSED*/
void XtGui::setCpuExitCallback(Widget w,
	XtPointer client_data, XtPointer call_data)
{
	if (client_data != NULL)
		((XtGui *)client_data)->set_new_state(S_EXIT);
}

/*ARGSUSED*/
void XtGui::setCpuRunCallback(Widget w,
	XtPointer client_data, XtPointer call_data)
{
	if (client_data != NULL)
		((XtGui *)client_data)->set_new_state(S_RUN);
}

/*ARGSUSED*/
void XtGui::setCpuStopCallback(Widget w,
	XtPointer client_data, XtPointer call_data)
{
	if (client_data != NULL)
		((XtGui *)client_data)->set_new_state(S_STOP);
}

/*ARGSUSED*/
void XtGui::setCpuStepCallback(Widget w,
	XtPointer client_data, XtPointer call_data)
{
	if (client_data != NULL)
		((XtGui *)client_data)->set_new_state(S_STEP);
}

/*ARGSUSED*/
void XtGui::setCpuNextCallback(Widget w,
	XtPointer client_data, XtPointer call_data)
{
	if (client_data != NULL)
		((XtGui *)client_data)->set_new_state(S_NEXT);
}

/*ARGSUSED*/
void XtGui::setCpuResetCallback(Widget w,
	XtPointer client_data, XtPointer call_data)
{
	if (client_data != NULL)
		((XtGui *)client_data)->set_new_state(S_RESET);
}

/*ARGSUSED*/
void XtGui::setCpuResetRunCallback(Widget w,
	XtPointer client_data, XtPointer call_data)
{
	if (client_data != NULL)
		((XtGui *)client_data)->set_new_state(S_RESET_RUN);
}

/*ARGSUSED*/
void XtGui::toggleCpuCallback(Widget w,
	XtPointer client_data, XtPointer call_data)
{
	if (client_data != NULL)
		((XtGui *)client_data)->toggle_cpu();
}

/*ARGSUSED*/
void XtGui::toggleUndocumentedCallback(Widget w,
	XtPointer client_data, XtPointer call_data)
{
	if (client_data != NULL)
		((XtGui *)client_data)->toggle_undocumented();
}

/*ARGSUSED*/
void XtGui::toggleFrequencyCallback(Widget w,
	XtPointer client_data, XtPointer call_data)
{
	if (client_data != NULL)
		((XtGui *)client_data)->toggle_frequency();
}

/*ARGSUSED*/
void XtGui::popdownCpuCallback(Widget w,
	XtPointer client_data, XtPointer call_data)
{
	if (client_data != NULL)
		((XtGui *)client_data)->popdown_cpu();
}

/*ARGSUSED*/
void XtGui::popupHelpCallback(Widget w,
	XtPointer client_data, XtPointer call_data)
{
	if (client_data != NULL)
		((XtGui *)client_data)->popup_help();
}

/*ARGSUSED*/
void XtGui::popdownHelpCallback(Widget w,
	XtPointer client_data, XtPointer call_data)
{
	if (client_data != NULL)
		((XtGui *)client_data)->popdown_help();
}

/*ARGSUSED*/
void XtGui::displayManPageCallback(Widget w,
	XtPointer client_data, XtPointer call_data)
{
	if (client_data != NULL)
		((XtGui *)client_data)->display_man_page(w);
}

/*ARGSUSED*/
void XtGui::popupAboutCallback(Widget w,
	XtPointer client_data, XtPointer call_data)
{
	if (client_data != NULL)
		((XtGui *)client_data)->popup_about();
}

/*ARGSUSED*/
void XtGui::popdownAboutCallback(Widget w,
	XtPointer client_data, XtPointer call_data)
{
	if (client_data != NULL)
		((XtGui *)client_data)->popdown_about();
}

#ifdef HAVE_XPM
/*ARGSUSED*/
void XtGui::popupAuthorCallback(Widget w,
	XtPointer client_data, XtPointer call_data)
{
	if (client_data != NULL) {
		((XtGui *)client_data)->popdown_about();
		((XtGui *)client_data)->popup_author();
	}
}

/*ARGSUSED*/
void XtGui::popdownAuthorCallback(Widget w,
	XtPointer client_data, XtPointer call_data)
{
	if (client_data != NULL)
		((XtGui *)client_data)->popdown_author();
}
#endif // iddef HAVE_XPM

/*ARGSUSED*/
void XtGui::popupBpCallback(Widget w,
	XtPointer client_data, XtPointer call_data)
{
	if (client_data != NULL)
		((XtGui *)client_data)->popup_bp();
}

/*ARGSUSED*/
void XtGui::popupLogfileCallback(Widget w,
	XtPointer client_data, XtPointer call_data)
{
	if (client_data != NULL)
		((XtGui *)client_data)->popup_log();
}

/*ARGSUSED*/
void XtGui::popdownLogCallback(Widget w,
	XtPointer client_data, XtPointer call_data)
{
	if (client_data != NULL)
		((XtGui *)client_data)->popdown_log(w);
}

/*ARGSUSED*/
void XtGui::clearLogCallback(Widget w,
	XtPointer client_data, XtPointer call_data)
{
	if (client_data != NULL)
		((XtGui *)client_data)->clear_log();
}

/*ARGSUSED*/
void XtGui::clearBpCallback(Widget w,
	XtPointer client_data, XtPointer call_data)
{
	if (client_data != NULL)
		((XtGui *)client_data)->clear_bp();
}

/*ARGSUSED*/
void XtGui::popdownBpCallback(Widget w,
	XtPointer client_data, XtPointer call_data)
{
	if (client_data != NULL)
		((XtGui *)client_data)->popdown_bp(w);
}

/*ARGSUSED*/
void XtGui::popdownMessageCallback(Widget w,
	XtPointer client_data, XtPointer call_data)
{
	if (client_data != NULL)
		((XtGui *)client_data)->popdown_message();
}

void XtGui::c_process_resize(XEvent *pevent)
{
	guiXSize = (pevent->xconfigure.width + 10) / WINDOWWIDTH;
	guiXSize = guiXSize < 1 ? 1 : guiXSize;
	guiYSize = (pevent->xconfigure.height + 10) / WINDOWHEIGHT;
	guiYSize = guiYSize < 1 ? 1 : guiYSize;
	e2video->init_blocks_to_update();
}

#ifdef HAVE_XPM
void XtGui::popup_author(void)
{
	XtPopup(authorframe, XtGrabExclusive);
}

void XtGui::a_expose(Widget w, XEvent *pevent)
{
	static int errorStatus = XpmSuccess;
	XpmAttributes xpmattr;

	if (authorpixmap == None && errorStatus == XpmSuccess) {
		xpmattr.colormap  = DefaultColormapOfScreen(XtScreen(w));
		xpmattr.valuemask = XpmColormap; 
		errorStatus = XpmCreatePixmapFromData(XtDisplay(w),
			XtWindow(w), author, &authorpixmap, NULL, &xpmattr);
		XpmFreeAttributes(&xpmattr);
		if (errorStatus != XpmSuccess)
			popup_message("\
sorry, unable to create image\n\
of author. (image need 62\n\
color entries)");
	}
	if (errorStatus == XpmSuccess)
		XCopyArea(XtDisplay(w), authorpixmap, XtWindow(w), e2gc,
			pevent->xexpose.x,     pevent->xexpose.y,
			pevent->xexpose.width, pevent->xexpose.height,
			pevent->xexpose.x,     pevent->xexpose.y);
}

void XtGui::popdown_author(void)
{
	XtPopdown(authorframe);
	if (authorpixmap != None) {
		XFreePixmap(XtDisplay(authorframe), authorpixmap);
		authorpixmap = None;
	}
}
#endif // #ifdef HAVE_XPM

void XtGui::popup_about(void)
{
	XtPopup(aboutframe, XtGrabNonexclusive);
	XtSetKeyboardFocus(aboutform, aboutbutton);
}

void XtGui::popdown_about(void)
{
	XtPopdown(aboutframe);
}

// return 0 on success
int XtGui::popup_help(void)
{
	if (XAbstractGui::popup_help() == -1)
		// unfinished
		; //XtPopup(helpframe, XtGrabNone);
	return 0;
}

void XtGui::popdown_help(void)
{
	XtPopdown(helpframe);
}

void XtGui::popup_message(char *pmessage)
{
	if (*pmessage)
		XtVaSetValues(messagetext, XtNstring, pmessage, NULL);
	XtPopup(messageframe, XtGrabExclusive);
}

void XtGui::popdown_message(void)
{
	XtPopdown(messageframe);
	if (exit_flag)
		set_new_state(S_EXIT);
}

void XtGui::toggle_frequency(void)
{
	float frequency;

	frequency_control_on = !frequency_control_on;
	frequency = frequency_control_on ? 1.3396 : 0.0;
	schedy->sync_exec(new CSetFrequency(*schedy, frequency));
	if (okpixmap != None)
		XtVaSetValues(entry27, XtNleftBitmap,
		frequency_control_on ? okpixmap : None, NULL);
}

void XtGui::toggle_undocumented(void)
{
	is_use_undocumented = !is_use_undocumented;
	cpu->set_use_undocumented(is_use_undocumented);
	if (okpixmap != None)
		XtVaSetValues(entry28, XtNleftBitmap,
		is_use_undocumented ? okpixmap : None, NULL);
}

void XtGui::timerCallback(XtIntervalId id)
{
	Window	root, child;
	int	newX, newY;
	int	winX, winY;
	unsigned int newButtonMask;
	int	revert;
	s_cpu_status stat;
	
	// check if program can be savely shut down
	if (schedy->is_finished())
	{
		delete cpu;
		delete this;
		delete schedy;
		exit(0);
	}
	// check if CPU view has to be updated
	if (schedy->get_status(stat))
	{
		update_cpuview(stat);
		if (okpixmap != None)
		{
			bool is_running = (stat.state == S_RUN ||
					stat.state == S_NEXT);
			XtVaSetValues(entry21, XtNleftBitmap,
			   is_running ? okpixmap : None, NULL);
			XtVaSetValues(entry22, XtNleftBitmap,
			   !is_running ? okpixmap : None, NULL);
		}
		if (stat.state == S_INVALID)
		{
			char err_msg[128];

        		sprintf((char *)&err_msg, "\
Got invalid instrucion\n\
pc=%04x instr=%02x %02x %02x %02x\n\
Processor stopped. To\n\
continue press Reset button",
			stat.pc, stat.instruction[0],
			stat.instruction[1], stat.instruction[2],
			stat.instruction[3]);
        		popup_message(err_msg);
		}
	}
	// update graphic display
	int display_block;
	for (display_block = 0; display_block < YBLOCKS; display_block++)
		update_block(display_block);
	
	XGetInputFocus(XtDisplay(e2toplevel), &child, &revert);
	if (child == XtWindow(e2toplevel) &&
		XQueryPointer(XtDisplay(e2screen),
		XtWindow(e2toplevel),
		&root, &child, &newX, &newY,
		&winX, &winY, &newButtonMask) == True) {
		io->put_joystick(newX - oldX, newY - oldY);
		io->put_joystick(convert_buttonmask(newButtonMask));
		oldX = newX;
		oldY = newY;
	} // if
	XAbstractGui::timerCallback();

	// re-register timer callback
	XtAppAddTimeOut(context, timebase, timerCallbackProc, this);
}

void XtGui::main_loop(void)
{
	XtAppMainLoop(context);
}


/* ARGSUSED */
void XtGui::c_expose(XEvent *pevent)
{

	if (pevent->xexpose.count == 0 && XtIsRealized(e2screen))
		e2video->init_blocks_to_update();
} // c_expose

void XtGui::c_keyPress(XEvent *pevent)
{
	SWord key;
	if ((key = translate_to_ascii(&pevent->xkey)) >= 0)
		io->put_ch(key);
} // c_keyPress

void XtGui::c_buttonPress(XEvent *pevent)
{
	// just catch event to prevent to get cached by the
	// window manager
} // c_buttonPress

void XtGui::c_focusIn(XEvent *pevent)
{
	// set SERPAR-Flag of Monitor to parallel
	// sorry, it's very dirty but it's implemented for
	// the most convenience of the user and it's ONLY
	// enabled if the default monitor file "neumon54.hex"
	// is used
	// output_to_graphic();
} // c_focusIn

void XtGui::c_clientMsg(XEvent *pevent)
{
	if (pevent->xclient.message_type == (unsigned long)atom[FLX_WM_PROTOCOLS] &&
	    pevent->xclient.data.l[0]    == (long)atom[FLX_WM_TAKE_FOCUS])
	{
		// Respond to Window Manager Protocol
		// with XSetInputFocus to get Focus
		XSetInputFocus(getDisplay(), getWindow(),
		RevertToParent, pevent->xclient.data.l[1]);
	}
} // c_clientMsg


void XtGui::initialize_e2window(struct sGuiOptions *pOptions)
{
	Widget w;

	frequency_control_on = false;
	is_use_undocumented  = false;
	initialize_conv_tables();
	w = create_main_view(pOptions->argc, pOptions->argv,
		pOptions->synchronized);
	create_message_dialog(w);
	create_about_dialog(w);
	//create_help_view(w); // man viewer not supported any more
#ifdef HAVE_XPM
	create_author_dialog(w);
#endif
	create_cpuview(w);
	create_bp_dialog(w);
	create_logfile_dialog(w);
	initialize_after_create(w, pOptions->inverse, pOptions->color);
	manage_widget(w);
	initialize_after_open(w, PROGRAMNAME);
	e2toplevel = w;
} // initialize_e2window

void XtGui::initializeWMProtocols()
{
	//int i, size = 0;
	//Atom atoms_return[maxAtoms];
	Atom wm_take_focus;

	/*for (i = 0; i < maxAtoms; i++)
		if (atomNames[i] != NULL)
			size++;
	if (!XInternAtoms(getDisplay(), const_cast<char **>(atomNames),
	   size, False, atoms_return))
	   fprintf(stderr, "Error in XInternAtoms\n");
	fprintf(stderr, "atom: %08x\n", atoms_return[0]);
	for (int i = 0; i < size; i++)
		atom[i] = atoms_return[i];
	if (!XSetWMProtocols(getDisplay(), getWindow(), &atom[FLX_WM_TAKE_FOCUS], 1))
		fprintf(stderr, "Error in XSetWMProtocols, size=%d\n", size);
	fprintf(stderr, "size=%d\n", size);
 	*/
	wm_take_focus = XInternAtom(getDisplay(), "WM_TAKE_FOCUS", False);
	XSetWMProtocols (getDisplay(), getWindow(),
                          &wm_take_focus, 1);
}

Widget XtGui::create_main_view(int argc, char *argv[], int synchronized)
{
	Widget mainview;

	XtSetLanguageProc(NULL, NULL, NULL); // set to default language

	// create widget tree:
#ifdef sun
	mainview = XtVaAppInitialize(&context, "Flexemu",
		opts, 0, (unsigned int *)&argc, argv,
		(const char **)fallback_resources, NULL);
#else
	mainview = XtVaAppInitialize(&context, "Flexemu",
		opts, 0, &argc, argv, fallback_resources, NULL);
#endif
	if (synchronized)
		XSynchronize(XtDisplay(mainview), True);
	XtAppAddConverter(context, XtRString, XtROrientation,
		XmuCvtStringToOrientation, NULL, 0);

	// use a Paned widget because it doesn't resize the menuBar
	form = XtVaCreateManagedWidget("form", panedWidgetClass, mainview,
		NULL);
	menubar = XtVaCreateManagedWidget("menuBar", boxWidgetClass, form,
		NULL);
	button1 = XtVaCreateManagedWidget("menuButton1", menuButtonWidgetClass,
		menubar, NULL);
	button2 = XtVaCreateManagedWidget("menuButton2", menuButtonWidgetClass,
		menubar, NULL);
	button3 = XtVaCreateManagedWidget("menuButton3", menuButtonWidgetClass,
		menubar, NULL);
	e2screen = XtVaCreateManagedWidget("screen", coreWidgetClass, form,
		XtNwidth, (XtArgVal)WINDOWWIDTH*guiXSize,
		XtNheight, (XtArgVal)WINDOWHEIGHT*guiYSize, NULL);
	menu1 = XtVaCreatePopupShell("menu1", simpleMenuWidgetClass, button1,
		NULL);
	menu2 = XtVaCreatePopupShell("menu2", simpleMenuWidgetClass, button2,
		NULL);
	menu3 = XtVaCreatePopupShell("menu3", simpleMenuWidgetClass, button3,
		NULL);
	entry12 = XtVaCreateManagedWidget("menuEntry12", smeBSBObjectClass,
		menu1, NULL);
	entry21 = XtVaCreateManagedWidget("menuEntry21", smeBSBObjectClass,
		menu2, NULL);
	entry22 = XtVaCreateManagedWidget("menuEntry22", smeBSBObjectClass,
		menu2, NULL);
	entry23 = XtVaCreateManagedWidget("menuEntry23", smeBSBObjectClass,
		menu2, NULL);
	line21  = XtVaCreateManagedWidget("line21", smeLineObjectClass,
		menu2, NULL);
	entry24 = XtVaCreateManagedWidget("menuEntry24", smeBSBObjectClass,
		menu2, NULL);
	entry25 = XtVaCreateManagedWidget("menuEntry25", smeBSBObjectClass,
		menu2, NULL);
	entry26 = XtVaCreateManagedWidget("menuEntry26", smeBSBObjectClass,
		menu2, NULL);
	line22 = XtVaCreateManagedWidget("line22", smeLineObjectClass,
		menu2, NULL);
	entry27 = XtVaCreateManagedWidget("menuEntry27", smeBSBObjectClass,
		menu2, NULL);
	entry28 = XtVaCreateManagedWidget("menuEntry28", smeBSBObjectClass,
		menu2, NULL);
	entry31 = XtVaCreateManagedWidget("menuEntry31", smeBSBObjectClass,
		menu3, NULL);
	entry32 = XtVaCreateManagedWidget("menuEntry32", smeBSBObjectClass,
		menu3, NULL);
	// must realize it now
	// to get valid contents of win
	XtRealizeWidget(mainview);
	initializeWMProtocols();
	SetColors(XtDisplay(mainview));
	Window win = XtWindow(mainview);
	XSetWindowColormap(XtDisplay(mainview), win, cmap);

	// add some menu callbacks:
	XtAddCallback(entry12, XtNcallback, setCpuExitCallback,
		(XtPointer)this);
	XtAddCallback(entry21, XtNcallback, setCpuRunCallback,
		(XtPointer)this);
	XtAddCallback(entry22, XtNcallback, setCpuStopCallback,
		(XtPointer)this);
	XtAddCallback(entry23, XtNcallback, setCpuResetRunCallback,
		(XtPointer)this);
	XtAddCallback(entry24, XtNcallback, toggleCpuCallback,
		(XtPointer)this);
	XtAddCallback(entry25, XtNcallback, popupBpCallback,
		(XtPointer)this);
	XtAddCallback(entry26, XtNcallback, popupLogfileCallback,
		(XtPointer)this);
	XtAddCallback(entry27, XtNcallback, toggleFrequencyCallback,
		(XtPointer)this);
	XtAddCallback(entry28, XtNcallback, toggleUndocumentedCallback,
		(XtPointer)this);
	XtAddCallback(entry31, XtNcallback, popupHelpCallback,
		(XtPointer)this);
	XtAddCallback(entry32, XtNcallback, popupAboutCallback,
		(XtPointer)this);
	return mainview;
}  // create_main_view


void XtGui::create_about_dialog(Widget parent)
{
	char			*aboutstring;

	aboutstring = XtMalloc(sizeof(PROGRAM_VERSION) + 2 + sizeof(HEADER1) +
		sizeof(HEADER2));

	if (aboutstring != NULL) {
		strcpy(aboutstring, HEADER1);
       		strcat(aboutstring, "V " PROGRAM_VERSION);
       		strcat(aboutstring, HEADER2);
       	} else {
       		aboutstring = "not enough memory";
       	}

	aboutframe = XtVaCreatePopupShell("about",
		transientShellWidgetClass, parent, NULL);
	aboutform = XtVaCreateManagedWidget("aboutForm",
		formWidgetClass, aboutframe, NULL);
	abouttext = XtVaCreateManagedWidget("aboutText", asciiTextWidgetClass,
		aboutform,
		XtNstring, (XtArgVal)aboutstring,
		XtNlength, (XtArgVal)(strlen(aboutstring)+1),
		NULL);
	aboutbutton = XtVaCreateManagedWidget("aboutButton",
		commandWidgetClass, aboutform,
#ifdef HAVE_XPM
		XtNhorizDistance, (XtArgVal)80, NULL);
#else
		XtNhorizDistance, (XtArgVal)100, NULL);
#endif

#ifdef HAVE_XPM
	morebutton = XtVaCreateManagedWidget("moreButton",
		commandWidgetClass, aboutform, NULL);
#endif

	// define some button callbacks:
	XtAddCallback(aboutbutton, XtNcallback, popdownAboutCallback,
		(XtPointer)this);
#ifdef HAVE_XPM
	XtAddCallback(morebutton, XtNcallback, popupAuthorCallback,
		(XtPointer)this);
#endif
	XtFree(aboutstring);
} // create_about_dialog

// the returned string must be released with XtFree()!!

String XtGui::get_man_page(char *pman_page_name)
{
	String	 phelpstring, ptmp;
	char	*p, ch1, ch2;
	char	*perr = "can't find man page ";
	FILE	*fp;
	char	 buffer[32];
	int	 count, error;

	error = 0;
	count = 0;
	ch1 = '\0';
	phelpstring = XtMalloc(8192);
	sprintf((char *)buffer, "man %s 2>&1", pman_page_name);
	if ((fp = popen(buffer , "r")) != NULL) {
		p = phelpstring;
		if (!feof(fp))
			ch1 = fgetc(fp);
		while (!feof(fp)) {
			ch2 = fgetc(fp);
			if (ch2 != 8 && ch1 != 8) { // 8 means ^H
				*(p++) = ch1;
				count++;
			}
			ch1 = ch2;
			if ((count & 0x1fff) == 0x1ffe) {
				if ((ptmp = XtMalloc(count + 8192)) != NULL) {
					memcpy(ptmp, phelpstring, count);
					p = ptmp + count;
					XtFree(phelpstring);
					phelpstring = ptmp;
				} else
					error = 1;
			} // if
		} // while
		// there are at least two bytes free in the stringbuffer
		if (ch1 >= ' ') 
			*(p++) = ch1;
		*p = '\0';
		if (pclose(fp) < 0)
			error = 2;
	} else
		error = 3;
	if (error || strlen(phelpstring) == 0) {
		strcpy(phelpstring, perr);
		strcat(phelpstring, pman_page_name);
	}  // if
	return phelpstring;
} // get_man_page

void XtGui::create_help_view(Widget parent)
{
	String phelpstring;
	char   buttonName[20];
	int    i;

	phelpstring = get_man_page("flexemu");
	helpframe = XtVaCreatePopupShell("help",
		transientShellWidgetClass, parent, NULL);
	helpform = XtVaCreateManagedWidget("helpForm",
		formWidgetClass, helpframe, NULL);
	helptext = XtVaCreateManagedWidget("helpText", asciiTextWidgetClass,
		helpform,
		XtNstring,	     (XtArgVal)phelpstring,
		XtNscrollVertical,   (XtArgVal)XawtextScrollWhenNeeded,
		XtNscrollHorizontal, (XtArgVal)XawtextScrollWhenNeeded,
		NULL);
	helpokbutton = XtVaCreateManagedWidget("helpButton",
		commandWidgetClass, helpform, NULL);

	for (i = 0; i < NR_OF_HELPBUTTONS; i++) {
		sprintf((char *)buttonName, "helpButton%d", i+1);
		helpbutton[i] = XtVaCreateManagedWidget(buttonName,
			commandWidgetClass, helpform, NULL);
	}
	XtFree(phelpstring);

	// define some button callbacks:
	XtAddCallback(helpokbutton,  XtNcallback, popdownHelpCallback,
		(XtPointer)this);
	for (i = 0; i < NR_OF_HELPBUTTONS; i++)
		XtAddCallback(helpbutton[i], XtNcallback,
			displayManPageCallback, (XtPointer)this);
} // create_help_view

void XtGui::display_man_page(Widget w)
{
	String plabel, phelpstring;
	char label[16];

	// the label of the button is the name of the man-page
	// so it can simply be changed by editing the resource file

	XtVaGetValues(w, XtNlabel, &plabel, NULL);
	strcpy((char *)&label, plabel);
	label[0] = tolower(label[0]); // lower first character
	phelpstring = get_man_page(label);
	XtVaSetValues(helptext, XtNstring, (XtArgVal)phelpstring, NULL);
	XtFree(phelpstring);
} // display_man_page


#ifdef HAVE_XPM
void XtGui::create_author_dialog(Widget parent)
{
	authorpixmap = None;
	authorframe = XtVaCreatePopupShell("author",
		transientShellWidgetClass, parent, NULL);
	authorform = XtVaCreateManagedWidget("authorForm",
		formWidgetClass, authorframe, NULL);
	authorwidget = XtVaCreateManagedWidget("authorWidget",
		coreWidgetClass, authorform, NULL);
	authorbutton = XtVaCreateManagedWidget("authorButton",
		commandWidgetClass, authorform, NULL);

	// define some button callbacks:
	XtAddCallback(authorbutton, XtNcallback, popdownAuthorCallback,
		(XtPointer)this);
} // create_author_dialog
#endif

void XtGui::create_message_dialog(Widget parent)
{
	messageframe = XtVaCreatePopupShell("message",
		transientShellWidgetClass, parent, NULL);
	messageform = XtVaCreateManagedWidget("messageForm",
		formWidgetClass, messageframe, NULL);
	messagetext = XtVaCreateManagedWidget("messageText",
		asciiTextWidgetClass, messageform, NULL);
	messagebutton = XtVaCreateManagedWidget("messageButton",
		commandWidgetClass, messageform, NULL);

	// define some button callbacks:
	XtAddCallback(messagebutton, XtNcallback, popdownMessageCallback,
		(XtPointer)this);
} // create_message_dialog

void XtGui::initialize_after_create(Widget w, int inverse, const char *color)
{
	unsigned int		i, j;
	Display		       *display;
	Screen		       *screen;
	Visual		       *visual;
	XGCValues		gcv;
	XWMHints               *wm_hints;
	XColor			xcolor, exact_color;
	static XtActionsRec	actions[] = {
#ifdef HAVE_XPM
					{"author_expose", (XtActionProc)author_expose},
#endif
					{"focusInMain", (XtActionProc)focusInMain},
					{"expose",   (XtActionProc)expose},
					{"keyPress", (XtActionProc)keyPress},
					{"buttonPress", (XtActionProc)buttonPress},
					{"convert", (XtActionProc)keyPress},
					{"popup_no_resources",
					   (XtActionProc)popup_no_resources},
					{"resize", (XtActionProc)processResize},
					{"clientMsg", (XtActionProc)clientMsg},
				};


	oldX       = 0;
	oldY       = 0;

	display = XtDisplay(w);
	screen  = XtScreen(w);
	visual  = DefaultVisualOfScreen(screen);

	mainpixmap = XCreateBitmapFromData(XtDisplay(w),
		RootWindowOfScreen(screen),
		flexmain_bits, flexmain_width, flexmain_height);
	if (mainpixmap != None)
		XtVaSetValues(w, XtNiconPixmap, mainpixmap, NULL);
	XtAppAddActions(context, actions, XtNumber(actions));

	if (!*color || !stricmp(color, "default"))
		color = "green";
	if (!XAllocNamedColor(display, DefaultColormapOfScreen(screen),
		color, &xcolor, &exact_color))
		xcolor.pixel = WhitePixelOfScreen(screen);
	if (!inverse) {
		gcv.foreground = xcolor.pixel;
		gcv.background = BlackPixelOfScreen(screen);
	} else {
		gcv.foreground = BlackPixelOfScreen(screen);
		gcv.background = xcolor.pixel;
	}
#ifdef HAVE_XPM
	authorgc = XtGetGC(e2screen, GCForeground | GCBackground, &gcv);
#endif
	e2gc = XtGetGC(e2screen, GCForeground | GCBackground, &gcv);
	XtVaSetValues(e2screen, XtNbackground, gcv.background, NULL);
	copy_block = new Byte[WINDOWWIDTH * BLOCKHEIGHT *
			MAX_GUIXSIZE * MAX_GUIYSIZE *
			32 / 8]; // max. screen depth
	// initialize different images used for different window sizes:
	for (i = 0; i < MAX_GUIXSIZE; i++) {
		for (j = 0; j < MAX_GUIYSIZE; j++) {
			image1[i][j] = XCreateImage(display, visual, 1,
				XYBitmap, 0, (char *)copy_block,
				BLOCKWIDTH*(i+1), BLOCKHEIGHT*(j+1), 32, 0);
			// use bitmap_unit = 8 to be independant of
			// byte_order
			image1[i][j]->bitmap_unit = 8;
			// Bit order on Eurocom II: always Bit 7 first
			image1[i][j]->bitmap_bit_order = MSBFirst;
			_XInitImageFuncPtrs(image1[i][j]);
			image6[i][j] = XCreateImage(display, visual, depth,
				ZPixmap, 0, (char *)copy_block,
				BLOCKWIDTH*(i+1), BLOCKHEIGHT*(j+1), 32, 0);
			switch (depth) {
			  case  1:
			  case  8: image6[i][j]->bitmap_unit = 8; break;
			  case 15:
			  case 16: image6[i][j]->bitmap_unit = 16; break;
			  case 24:
			  case 32: image6[i][j]->bitmap_unit = 32; break;
			}
			image6[i][j]->bitmap_bit_order = MSBFirst;
#ifdef WORDS_BIGENDIAN
			image6[i][j]->byte_order = MSBFirst;
#else
			image6[i][j]->byte_order = LSBFirst;
#endif
			_XInitImageFuncPtrs(image6[i][j]);
		} // for
	} // for

        wm_hints = XAllocWMHints();
	wm_hints->initial_state = NormalState;
	wm_hints->input         = True;
	wm_hints->icon_pixmap	= mainpixmap;
	wm_hints->flags = StateHint | IconPixmapHint | InputHint;
	XSetWMHints(XtDisplay(w), XtWindow(w), wm_hints);
	XFree(wm_hints);
	XtAppAddTimeOut(context, timebase, timerCallbackProc, this);

	// needed for propper expose:
	XtInstallAllAccelerators(e2screen, form);
#ifdef HAVE_XPM
	XtInstallAllAccelerators(authorwidget, authorframe);
#endif
}

void XtGui::manage_widget(Widget w)
{
	s_cpu_status stat;
	stat.mnemonic[0] = '\0';
	XtRealizeWidget(w);
	XtMapWidget(w);
	e2video->init_blocks_to_update();	// update main view
	update_cpuview(stat);
}  // manage widget


void XtGui::initialize_after_open(Widget w, char *title)
{
	Dimension	width, height;
	XSizeHints	*pxsh;

	set_cursor();
	okpixmap = XCreateBitmapFromData(XtDisplay(w),
		RootWindowOfScreen(XtScreen(w)), ok_bits, ok_width, ok_height);
	if (okpixmap != None && cpu->is_use_undocumented())
		XtVaSetValues(entry28, XtNleftBitmap, okpixmap, NULL);
	XtVaGetValues(w, XtNwidth, &width, XtNheight, &height, NULL);
	pxsh = XAllocSizeHints();
	if (pxsh != NULL) {
		pxsh->min_width  = WINDOWWIDTH + (width % WINDOWWIDTH);
		pxsh->max_width  = WINDOWWIDTH * MAX_GUIXSIZE +
			(width % WINDOWWIDTH);
		pxsh->min_height = WINDOWHEIGHT + (height % WINDOWHEIGHT);;
		pxsh->max_height = WINDOWHEIGHT * MAX_GUIYSIZE +
			(height % WINDOWHEIGHT);
		pxsh->width_inc  = WINDOWWIDTH;
		pxsh->height_inc = WINDOWHEIGHT;
		pxsh->win_gravity= NorthWestGravity;
		pxsh->flags      = PMaxSize | PMinSize | PResizeInc |
                                   PWinGravity;
		XSetStandardProperties(XtDisplay(w), XtWindow(w),
			title, title, None, NULL, 0, pxsh);
		XFree(pxsh);
	}
}

Display *XtGui::getDisplay(void) {
	return XtDisplay(e2screen);
}

Window XtGui::getWindow(void) {
	return XtWindow(e2screen);
}

int XtGui::gui_type(void)
{
	return GUI_XTOOLKIT;
}

XtGui::XtGui(
	Mc6809*    x_cpu,
	Scheduler* x_sched,
	Inout*     x_io,
	E2video*   x_video,
	struct sGuiOptions *pOptions) :
		XAbstractGui(x_cpu, x_sched, x_io, x_video, pOptions)
{
	initialize(pOptions);
}

XtGui::~XtGui()
{
	int i, j;

	if (okpixmap != None)
		XFreePixmap(XtDisplay(e2toplevel), okpixmap);
	if (mainpixmap != None)
		XFreePixmap(XtDisplay(e2toplevel), mainpixmap);
	if (cpupixmap != None)
		XFreePixmap(XtDisplay(e2toplevel), cpupixmap);
	// remove data pointer otherwise it would be freed
	for (i = 0; i < MAX_GUIXSIZE; i++) {
		for (j = 0; j < MAX_GUIYSIZE; j++) {
			image1[i][j]->data = NULL;
			XDestroyImage(image1[i][j]);
			image6[i][j]->data = NULL;
			XDestroyImage(image6[i][j]);
		} // for
	} // for
#ifdef HAVE_XPM
	if (authorpixmap != None)
		XFreePixmap(XtDisplay(e2toplevel), authorpixmap);
	XtReleaseGC(e2screen, authorgc);
#endif
	XtReleaseGC(e2screen, e2gc);
	if (own_cmap)
		XFreeColormap(getDisplay(), cmap);
	XtDestroyApplicationContext(context);
	delete [] copy_block;
	copy_block = NULL;
}

#endif // #ifdef HAVE_XTK

